﻿using Hotline.FlowEngine.Workflows;
using Hotline.Settings;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using MapsterMapper;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.KnowledgeBase
{
    public class KnowledgeDomainService : IKnowledgeDomainService, IScopeDependency
    {
        private readonly IKnowledgeRepository _knowledgeRepository;
        private readonly IKnowledgeTempRepository _knowledgeTempRepository;
        private readonly IMapper _mapper;
        private readonly IKnowledgePVRepository _knowledgePVRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="knowledgeRepository"></param>
        /// <param name="knowledgeTempRepository"></param>
        /// <param name="mapper"></param>
        /// <param name="knowledgePVRepository"></param>
        public KnowledgeDomainService(IKnowledgeRepository knowledgeRepository, IKnowledgeTempRepository knowledgeTempRepository, IMapper mapper, IKnowledgePVRepository knowledgePVRepository)
        {
            _knowledgeRepository = knowledgeRepository;
            _mapper = mapper;
            _knowledgeTempRepository = knowledgeTempRepository;
            _knowledgePVRepository = knowledgePVRepository;
        }

        /// <summary>
        /// 知识库-查询详情-增加浏览量
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<Knowledge> KnowledgeInfo(string Id, CancellationToken cancellationToken)
        {
            var know = await _knowledgeRepository.GetAsync(Id, cancellationToken);
            if (know is null)
                throw UserFriendlyException.SameMessage("知识查询失败！");
            return know;
        }

        /// <summary>
        /// 浏览记录写入
        /// </summary>
        /// <param name="know"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task KnowledgePvIncreaseAsync(Knowledge know, CancellationToken cancellationToken)
        {
            //写入浏览记录
            KnowledgePv knowledgePv = new()
            {
                CreationTime = DateTime.Now,
                KnowledgeCode = know.Code
            };
            //浏览记录写入
            await _knowledgePVRepository.AddAsync(knowledgePv, cancellationToken);

            //修改浏览量
            know.PageView++;
            //修改点击量
            await _knowledgeRepository.UpdateAsync(know, cancellationToken);
        }

        /// <summary>
        /// 知识库-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<string> KnowledgeAdd(AddKnowledgeDto dto, CancellationToken cancellationToken)
        {
            var kn = _mapper.Map<Knowledge>(dto);
            //知识草稿ID如果不为空，则表示是草稿箱的数据
            if (!string.IsNullOrEmpty(dto.Code))
            {
                //移除草稿箱数据
                await _knowledgeTempRepository.RemoveAsync(p => p.Code == dto.Code, false, cancellationToken);
            }
            //如果草稿箱不存在数据，也就是Code为空，从新生成Code
            if (string.IsNullOrEmpty(kn.Code))
                kn.Code = Convert.ToInt64((DateTime.Now - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds).ToString();
            kn.Status = EKnowledgeStatus.Auditing;

            return await _knowledgeRepository.AddAsync(kn, cancellationToken);
        }

        /// <summary>
        /// 知识库-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<string> KnowledgeUpdate(UpdateKnowledgeDto dto, CancellationToken cancellationToken)
        {
            //知识修改逻辑
            //如果可以修改，此条数据不动，重新写入一条新数据，知识编号不变，版本号+1，审批通过以后在数据交换

            var know = await _knowledgeRepository.GetAsync(dto.Id, cancellationToken);
            if (know is null || know.IsDeleted)
                throw UserFriendlyException.SameMessage("无效知识");
            //审批中数据不能处理
            if (know.Status == EKnowledgeStatus.Auditing)
                throw UserFriendlyException.SameMessage("审批中数据不能修改");

            var knowledge = _mapper.Map<Knowledge>(dto);
            knowledge.Code = know.Code;
            knowledge.PageView = know.PageView;
            knowledge.Status = EKnowledgeStatus.Auditing;
            knowledge.Id = "";
            knowledge.Version = 0;
            //新增一条数据
            return await _knowledgeRepository.AddAsync(knowledge, cancellationToken);

        }

        /// <summary>
        /// 流程审批完成，处理知识状态
        /// </summary>
        /// <param name="workflow"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        public async Task EndWorkKnowledge(Workflow workflow, CancellationToken cancellationToken)
        {
            var knowledge = await _knowledgeRepository.GetAsync(workflow.ExternalId, cancellationToken);
            if (knowledge == null)
                throw new UserFriendlyException($"知识查询失败, workflowId: {workflow.Id}", "无效流程编号");
            switch (workflow.ModuleCode)
            {
                case WorkflowModuleConsts.KnowledgeAdd://新增
                    //修改知识状态为已上架
                    knowledge.Status = EKnowledgeStatus.OnShelf;
                    knowledge.OnShelfTime = System.DateTime.Now;
                    await _knowledgeRepository.UpdateAsync(knowledge);
                    break;
                case WorkflowModuleConsts.KnowledgeUpdate://修改
                    //处理知识版本号
                    //根据Code查询目前使用的版本号，版本号+1
                    var oldKnowledge = await _knowledgeRepository
                        .Queryable()
                        .Where(p => p.Code == knowledge.Code && p.Status != EKnowledgeStatus.Auditing && p.Status != EKnowledgeStatus.Failed && p.IsDeleted == false)
                        .OrderByDescending(p => p.Version)
                        .Take(1)
                        .FirstAsync();
                    if (oldKnowledge == null)
                        knowledge.Version = 1;
                    else
                        knowledge.Version = oldKnowledge.Version + 1;
                    //修改知识状态为已上架
                    knowledge.Status = EKnowledgeStatus.OnShelf;
                    knowledge.OnShelfTime = System.DateTime.Now;
                    await _knowledgeRepository.UpdateAsync(knowledge);
                    // await KnowledgeUpdate(knowledge);
                    break;

                case WorkflowModuleConsts.KnowledgeDelete://删除
                    ////知识先下架，在删除                                       
                    //knowledge.Status = EKnowledgeStatus.OffShelf;
                    //knowledge.OffShelfTime = System.DateTime.Now;
                    //await _knowledgeRepository.UpdateAsync(knowledge);
                    //删除知识
                    await _knowledgeRepository.RemoveAsync(p => p.Code == knowledge.Code, true, cancellationToken);
                    break;
            }
        }

        /// <summary>
        /// 交换数据、为了保持浏览记录等数据的存在
        /// </summary>
        /// <param name="knowledge"></param>
        /// <returns></returns>
        private async Task KnowledgeUpdate(Knowledge knowledge)
        {
            var Version = 1;
            //处理知识版本号
            //根据Code查询目前使用的版本号，版本号+1
            var oldKnowledge = await _knowledgeRepository
                .Queryable()
                .Where(p => p.Code == knowledge.Code && p.Status != EKnowledgeStatus.Auditing && p.Status != EKnowledgeStatus.Failed && p.IsDeleted == false)
                .OrderByDescending(p => p.Version)
                .Take(1)
                .FirstAsync();
            //如果没查到历史版本，直接改当前的数据
            if (oldKnowledge != null)
            {
                Version = oldKnowledge.Version + 1;   //已经查询到版本

                //交换数据、为了保持浏览记录等数据的存在
                var tempId = knowledge.Id;//最新审批通过的ID
                var tempOldId = oldKnowledge.Id;//原始数据ID
                var temp = oldKnowledge;//保存原始数据

                _mapper.Map(knowledge, oldKnowledge);//最新数据赋值给原始数据
                oldKnowledge.Id = tempOldId;//ID不变
                oldKnowledge.Status = EKnowledgeStatus.OnShelf;//改变知识状态
                oldKnowledge.OnShelfTime = System.DateTime.Now;//自动上架时间
                oldKnowledge.OffShelfTime = null;
                oldKnowledge.Version = Version;

                _mapper.Map(temp, knowledge);
                knowledge.Id = tempId;

                //修改数据
                await _knowledgeRepository.UpdateAsync(knowledge);
                await _knowledgeRepository.UpdateAsync(oldKnowledge);
            }
            else
            {
                knowledge.Status = EKnowledgeStatus.OnShelf;
                knowledge.OnShelfTime = System.DateTime.Now;
                knowledge.OffShelfTime = null;
                knowledge.Version = Version;
                await _knowledgeRepository.UpdateAsync(knowledge);
            }
        }

    }
}